@@ -2,7 +2,8 @@
 #
 # spigot is a rate limiter for aggregating syndicated content to pump.io
 #
-# (c) 2011-2013 by Nathan D. Smith <nathan@smithfam.info>
+# (c) 2011-2014 by Nathan D. Smith <nathan@smithfam.info>
+# (c) 2014 Craig Maloney <craig@decafbad.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +21,6 @@
 # Standard library imports
 import argparse
 from datetime import datetime, timedelta
-import hashlib
 try:
     import json
 except ImportError:
@@ -38,12 +38,13 @@ import feedparser
 from pypump import PyPump
 from pypump import Client
 
-
+# Helper functions
 def simple_verifier(url):
     print 'Please follow the instructions at the following URL:'
     print url
     return raw_input("Verifier: ")
 
+#Spigot classes
 
 class SpigotConfig(dict):
     """Extends the built-in dict type to provide a configuration interface for
@@ -67,18 +68,26 @@ class SpigotConfig(dict):
             self.update(json.loads(open(self.config_file, "r").read()))
         except IOError:
             logging.warning("Could not load configuration file")
+        
+        # Check for pre-2.2 formatted spigot configuration file
+        if not self.no_config:
+            formats = [self["feeds"][feed]["format"] for feed in self["feeds"]]
+            for format in formats:
+                if ( ("$t" in format) or ("$l" in format) ):
+                    logging.error("Existing config not upgraded for Spigot 2.2")
+                    logging.error("Please upgrade the config using the utils/convert.py script found in the source repository.")
+                    sys.exit(2)
 
     def save(self):
         "Convert the state of the SpigotConfig dict to json and save."
 
-        logging.debug("Saving spigot.json")
+        logging.debug("Saving %s" % self.config_file)
         try:
             open(self.config_file, "w").write(json.dumps(self, indent=4))
         except IOError:
             logging.exception("Could not save configuration file")
             sys.exit(2)
 
-
     def add_user(self):
         "Interactively add a new user to the configuration."
 
@@ -126,6 +135,13 @@ class SpigotConfig(dict):
 
         print "Adding feed..."
         url = raw_input("Feed URL: ")
+        # Test feed for presence, validity
+        test_feed = None
+        try:
+            test_feed = feedparser.parse(url)
+            logging.debug("Successfully parsed feed %s" % url)
+        except:
+            logging.warning("Could not parse feed %s" % url)
         accounts = self["accounts"].keys()
         print "Choose an account:"
         for i in range(len(accounts)):
@@ -149,11 +165,16 @@ class SpigotConfig(dict):
                 interval = int(raw_inter)
                 valid_interval = True
             except:
-                print "Invalide interval specified."
+                print "Invalid interval specified."
         print """Spigot formats your outgoing posts based on fields in the feed
-              being scanned. Use the following substitutions:
-              $t - title
-              $l - link"""
+              being scanned. Specify the field name surrounded by the '%'
+              character to have it replaced with the corresponding value for the
+              item (e.g. %title% or %link)."""
+        if test_feed:
+            print """The following fields are present in an example item in
+                     this feed:"""
+            for field in test_feed["items"][0].keys():
+                print field
         form = raw_input("Format: ")
         
         # Put it all together
@@ -192,6 +213,7 @@ class SpigotConfig(dict):
             logging.debug("  Added to list of feeds to poll")
         return feeds_to_poll
 
+
 class SpigotDB():
     """Handle database calls for Spigot."""
  
@@ -216,14 +238,25 @@ class SpigotDB():
             
         if new_db:
                 self._init_db_tables()
-                
+        
+        # Test for pre-2.2 database structure
+        if not new_db:
+            curs = self._db.cursor()
+            curs.execute("PRAGMA table_info(items);")
+            cols = curs.fetchall()
+            curs.close()
+            if not "message" in [col[1] for col in cols]:
+                logging.error("Existing database not upgraded for Spigot 2.2")
+                logging.error("Please upgrade the database using the utils/convert.py script found in the source repository.")
+                sys.exit(2)
+
     def _init_db_tables(self):
         """Initialize the database if it is new"""
 
         curs = self._db.cursor()
         # Figure out db tables based on tricklepost
-        create_query = """create table items (feed text, link text, title text,
-            hash text, date timestamp, posted timestamp)"""
+        create_query = """create table items (feed text, link text,
+                          message text, date timestamp, posted timestamp)"""
         curs.execute(create_query)
         self._db.commit()
         logging.debug("Initialized database tables")
@@ -235,25 +268,25 @@ class SpigotDB():
         self._db.close()
         logging.debug("Closed connection to database")
         
-    def check_hash(self, item_hash):
-        """Returns true if the specified hash is already in the database."""
+    def check_link(self, item_link):
+        """Returns true if the specified link is already in the database."""
         
         curs = self._db.cursor()
-        curs.execute("select * from items where hash=?", [item_hash])
+        curs.execute("select * from items where link=?", [item_link])
         if len(curs.fetchall()) > 0:
             return True
         else:
             return False
         curs.close()
             
-    def add_item(self, feed_url, link, title, item_hash, date):
+    def add_item(self, feed_url, link, message, date):
         """Add an item to the database with the given parameters. Return True
         if successful."""
         
         curs = self._db.cursor()
-        curs.execute("insert into items(feed, link, title, hash, date) \
-            values (?, ?, ?, ?, ?)", (feed_url, link, title, item_hash, date))
-        logging.debug("    Added item %s to database" % item_hash)
+        curs.execute("insert into items(feed, link, message, date) \
+            values (?, ?, ?, ?)", (feed_url, link, message, date))
+        logging.debug("    Added item %s to database" % link)
         curs.close()
         
         self._db.commit()
@@ -263,7 +296,7 @@ class SpigotDB():
         through to the specified statusnet account."""
         
         curs = self._db.cursor()
-        curs.execute("SELECT * FROM items where (posted is NULL AND feed=?) \
+        curs.execute("SELECT feed, link, message FROM items where (posted is NULL AND feed=?) \
             ORDER BY date ASC",[feed])
         unposted_items = curs.fetchall()
         num_items = len(unposted_items)
@@ -271,16 +304,16 @@ class SpigotDB():
         curs.close()
         return unposted_items
 
-    def mark_posted(self, item_hash, date=None):
+    def mark_posted(self, item_link, date=None):
         """Mark the given item posted by setting its posted datetime to now."""
         
         if not date:
             date = datetime.utcnow()
         curs = self._db.cursor()
-        curs.execute("UPDATE items SET posted=? WHERE hash=?",
-            (date, item_hash))
+        curs.execute("UPDATE items SET posted=? WHERE link=?",
+            (date, item_link))
         logging.debug("  Updated posted time of item %s in database"
-            % item_hash)
+            % item_link)
         curs.close()
         self._db.commit()
     
@@ -313,6 +346,35 @@ class SpigotFeeds():
         self._spigotdb = db
         self._config = config
 
+    def format_message(self, feed, entry):
+        """Returns an outgoing message for the given entry based on the given 
+        feed's configured format."""
+
+        message = self._config["feeds"][feed]["format"]
+        # Store a list of tuples containing format string and value
+        replaces = []
+        field_re = re.compile("%\w+%")
+        fields = field_re.findall(message)
+        for raw_field in fields:
+            # Trim the % character from format
+            field = raw_field[1:-1]
+            if field in entry:
+                # Make a special exception for the content element, which in
+                # ATOM can appear multiple times per entry. Assume the element
+                # with index=0 is the desired value.
+                if field == "content":
+                    logging.debug("    'content' field in formatting string")
+                    value = entry.content[0].value
+                else:
+                    value = entry[field]
+            else:
+                value = ""
+            replaces.append( (raw_field, value) )
+        # Fill in the message format with actual values
+        for string, val in replaces:
+            message = message.replace(string, val)
+        return message
+
     def poll_feeds(self):
         """Check the configured feeds for new posts."""
         
@@ -333,9 +395,6 @@ class SpigotFeeds():
         # Get a list of items for the feed and compare it to the database
         num_items = len(p.entries)
         logging.debug("Found %d items in feed %s" % (num_items,url))
-        # Find out which encoding the feed uses to avoid problems with hashlib
-        # below
-        enc = p.encoding
         new_items = 0
         for i in range(len(p.entries)):
             logging.debug("  Processing item %d" % i)
@@ -350,19 +409,14 @@ class SpigotFeeds():
                 date = p.entries[i].updated_parsed
             date_struct = datetime.fromtimestamp(mktime(date))
             logging.debug("    Date: %s" % datetime.isoformat(date_struct))
-            # Create a md5 hash of title and link, so that we can
-            # easily tell duplicates in the feed. Could use guid for this, but
-            # I don't trust feeds to use it correctly.
-            h = hashlib.md5()
-            hash_input = "%s|%s" % (title, link)
-            h.update(hash_input.encode(enc))
-            item_hash = h.hexdigest()
-            logging.debug("    Hash: %s" % item_hash)
+            logging.debug("    Link: %s" % link)
+            # Craft the message based feed format string
+            message = self.format_message(url,p.entries[i])
+            logging.debug("    Message: %s" % message)
             # Check to see if item has already entered the database
-            if not self._spigotdb.check_hash(item_hash):
+            if not self._spigotdb.check_link(link):
                 logging.debug("    Not in database")
-                self._spigotdb.add_item(url, link, title, item_hash,
-                    date_struct)
+                self._spigotdb.add_item(url, link, message, date_struct)
                 new_items += 1
             else:
                 logging.debug("    Already in database")
@@ -402,18 +456,6 @@ class SpigotPost():
         self._config = spigot_config
         self._spigotfeed = spigot_feed
 
-    def _format_message(self, feed, link, title, form):
-        # TODO Allow arbitrary insertion of feedparser properties between %%
-        """Return a string formatted according to the feed's configuration.
-        
-        Replacement strings:
-        $t : title
-        $l : link"""
-        
-        message = form.replace("$t",title)
-        message = message.replace("$l",link)
-        return message
-
     def post_items(self):
         """Handle the posting of unposted items.
         
@@ -449,18 +491,17 @@ class SpigotPost():
                 except:
                     # Escape the loop if there are no new posts waiting
                     break
+                feed = item[0]
                 link = item[1]
-                title = item[2]
-                item_hash = item[3]
-                message = self._format_message(feed, link, title, form)
+                message = item[2]
 
                 try:
                     logging.info("  Posting item %s from %s to account %s" 
-                                 % (item_hash,feed,account))
+                                 % (link,feed,account))
                     new_note = pump.Note(message)
                     new_note.to = pump.Public
                     new_note.send()
-                    self._spigotdb.mark_posted(item_hash)
+                    self._spigotdb.mark_posted(link)
                 except:
                     logging.exception("  Unable to post item")
 
